pub trait ButtonSpec: Copy + Into<u8> {}
impl<T: Copy + Into<u8>> ButtonSpec for T {}

#[derive(Clone, Copy, Debug)]
pub struct ButtonSet<BS: ButtonSpec>(u64, std::marker::PhantomData<BS>);

impl<BS: ButtonSpec> ButtonSet<BS> {
    pub fn active(&self, button: BS) -> bool {
        (self.0 >> (button.into())) & 1 == 1
    }
    pub fn inactive(&self, button: BS) -> bool {
        !self.active(button)
    }

    pub fn set_button(&mut self, button: BS, to: bool) {
        if to {
            self.0 |= 1 << button.into();
        } else {
            self.0 &= !(1 << button.into());
        }
    }
}

impl<BS: ButtonSpec> Default for ButtonSet<BS> {
    fn default() -> Self {
        Self(0, Default::default())
    }
}

impl<BS: ButtonSpec> std::ops::BitAnd for ButtonSet<BS> {
    type Output = Self;
    fn bitand(self, rhs: Self) -> Self::Output {
        Self(self.0 & rhs.0, Default::default())
    }
}

impl<BS: ButtonSpec> std::ops::BitOr for ButtonSet<BS> {
    type Output = Self;
    fn bitor(self, rhs: Self) -> Self::Output {
        Self(self.0 | rhs.0, Default::default())
    }
}

impl<BS: ButtonSpec> std::ops::Not for ButtonSet<BS> {
    type Output = Self;
    fn not(self) -> Self::Output {
        Self(!self.0, Default::default())
    }
}
