use crate::text::TextLine;

use kahlo::colour::Colour;

pub struct Theme {
    pub active: Colour,
    pub border: Colour,
    pub border_width: usize,
    pub background: Colour,
    pub panel: Colour,
    pub foreground: Colour,

    pub ui_font: fontdue::Font,
    pub ui_font_size: f32,
}

impl Theme {
    pub fn make_line<'l>(&'l self, text: &'l str) -> TextLine<'l> {
        TextLine::new(&self.ui_font, text, self.ui_font_size)
    }

    pub fn default_with_font(ui_font: fontdue::Font) -> Self {
        Self {
            active: Colour::hex_rgb("#c16e70").unwrap(),
            border: Colour::hex_rgb("#dc9e82").unwrap(),
            border_width: 3,
            background: Colour::hex_rgb("#030027").unwrap(),
            panel: Colour::hex_rgb("#151e3f").unwrap(),
            foreground: Colour::WHITE,

            // other colour in palette: #F2F3D9
            ui_font,
            ui_font_size: 16.0,
        }
    }
}
